<?php

namespace Modules\CADPED\Controllers;

class Cliente {
    protected $connection;
    protected $Utils;

    public function __construct($connection, $UtilsIntegracao){
        $this->connection = $connection;
        $this->Utils = $UtilsIntegracao;
    }

    /**
     * www.apiteknisa.com/cadPed/cliente?IDCLIEATIV=A&DTINCLUSAO=01/01/2000&CDEXCLIENTE=
    */

    public function endpointGetClientes($request, $response) {
        try{
            $conditions = '';
            $idclieativ = strtoupper($this->Utils->parametro($request, 'IDCLIEATIV'));
            $dtinclusao = $this->Utils->parametro($request, 'DTINCLUSAO');
            $cdexcliente = $this->Utils->parametro($request, 'CDEXCLIENTE');

            if ($idclieativ != 'S' && $idclieativ != 'N' && $idclieativ != 'A') $this->Utils->incluiMensagem('Tipo de situação informado não é um valor válido. Espera-se: S (Ativo), N (Inativo) ou A (Ambos)', true);
            
            if ($idclieativ === 'A') $idclieativ = 'S_N_';
            
            if(empty($dtinclusao)) $this->Utils->incluiMensagem('Data não informada', true);
            
            if(!$this->Utils->dataValida($dtinclusao)) $this->Utils->incluiMensagem("Data informada não é um valor válido", true);
            
            if($this->Utils->statusAtual() == 'E') {
                $this->Utils->retornoJSONStatusMensagem($response);
                return;    
            }
            
            if ($cdexcliente) {
                $conditions .= ' AND CDEXCLIENTE = :CDEXCLIENTE';
                $paramExCliente = array(
                  "CDEXCLIENTE" => $cdexcliente
                );
            }
            
            $query = "SELECT C.IDTPIJURCLIE, C.NRINSJURCLIE, C.NMRAZSOCCLIE, C.NMFANTCLIE, C.IDCLIEATIV, TO_CHAR(C.DTINCLUSAO, 'DD/MM/YYYY') AS DTINCLUSAO, 
                        C.CDEXCLIENTE, EC.IDTPENDECLIE, EC.DSENDECLIE, EC.CDBAIRRO, EC.NMBAIRCLIE, EC.DSCOMPENDCLI, 
                        REPLACE(REPLACE(EC.NRCEPCLIE, '.', ''), '-', '') AS NRCEPCLIE, EC.CDPAIS, P.NMPAIS, EC.SGESTADO, E.NMESTADO, EC.CDMUNICIPIO, M.NMMUNICIPIO,
                        EC.DSPTOREFECLI, EC.NMCONTCLIE, EC.NRTELECLIE, EC.NRRAMALCLIE, EC.NRTELECLIE1,
                        EC.NRRAMALCLIE1, EC.NRFAXCLIE, EC.NRTELRESICLI, EC.NRCELULARCLI, EC.DSEMAILCLIE,
                        EC.DSEMAILCLIE2
                      FROM CLIENTE C
                      LEFT JOIN ENDECLIE EC ON EC.IDTPENDECLIE = 'P' AND EC.CDCLIENTE = C.CDCLIENTE
                     INNER JOIN PAIS      P ON P.CDPAIS = EC.CDPAIS
                     INNER JOIN ESTADO    E ON E.CDPAIS = EC.CDPAIS
                       AND E.SGESTADO = EC.SGESTADO
                      LEFT JOIN MUNICIPIO M ON M.CDPAIS = EC.CDPAIS
                       AND M.SGESTADO = EC.SGESTADO
                       AND M.CDMUNICIPIO = EC.CDMUNICIPIO 
                      /*LEFT JOIN BAIRRO    B ON B.CDBAIRRO = EC.CDBAIRRO
                       AND B.NMBAIRRO = EC.NMBAIRCLIE
                       AND B.CDMUNICIPIO = EC.CDMUNICIPIO
                       AND B.CDPAIS = EC.CDPAIS
                       AND B.SGESTADO = EC.SGESTADO*/
                     WHERE INSTR(:IDCLIEATIV, C.IDCLIEATIV) > 0
                       AND C.DTINCLUSAO >= :DTINCLUSAO ". 
                        $conditions."
                     ORDER BY C.NRINSJURCLIE";
            
            $params = [
                "IDCLIEATIV" => $idclieativ,
                "DTINCLUSAO" => $dtinclusao
            ];
            
            if ($cdexcliente) $params = array_merge($paramExCliente, $params);
            
            // var_dump($params);die;
            
            $prepared = $this->connection->prepare($query);
            $prepared->execute($params);
            $data = $prepared->fetchAll();
            $this->Utils->retornoJSON($response, $data);
        }catch(\Exception $e){
            if($this->Utils->modoDesenvolvedor())
                throw $e;
            $this->Utils->incluiMensagem("Ocorreu um erro não esperado na execução da rotina. Verifique os parâmetros e tente novamente", true);
            $this->Utils->retornoJSONStatusMensagem($response);
        }
    }
}