<?php
/**
 * Created by JetBrains PhpStorm.
 * User: geovanesantos
 * Date: 19/08/14
 * Time: 10:56
 * To change this template use File | Settings | File Templates.
 */

namespace Scheduler\Logic;

include_once "MailSender.php";
/**
 * Class Report
 * @package Scheduler\Logic
 * Representa um relatório do Birt pronto, mantendo a PK do relatório em PAINEL, parâmetros, lingua, e o caminho
 * para o relatório, após gerado.
 */
class Report {

    const REPORT_FORMAT_HTML="HTML";
    const REPORT_FORMAT_PDF="PDF";
    const REPORT_FORMAT_XLS="XLS";

    private $reportName;
    private $locale='pt_BR';
    private $reportPath;
    private $reportFormat;
    private $inputtedParameters;
    private static $REPOSITORY_PATH;

    public function setReportFormat($reportFormat)
    {
        $this->reportFormat = $reportFormat;
    }

    public function getReportFormat()
    {
        $format=$this->reportFormat;
        if($format===null) $format=self::REPORT_FORMAT_PDF;
        return $format;
    }

    public static function setRepositoryPath($REPOSITORY_PATH)
    {
        self::$REPOSITORY_PATH = $REPOSITORY_PATH;
    }

    public static function getRepositoryPath()
    {
        return self::$REPOSITORY_PATH;
    }

    public function setInputtedParameters($inputtedParameters)
    {
        $this->inputtedParameters = $inputtedParameters;
    }

    public function getInputtedParameters()
    {
        return $this->inputtedParameters;
    }

    public function addParameter($name, $value){
        $this->inputtedParameters[$name]=$value;
    }

    public function addParameterArray(Array $parameterArray){
        $this->inputtedParameters=array_merge($this->inputtedParameters, $parameterArray);
    }

    public function setReportPath($reportPath)
    {
        $this->reportPath = realpath(str_replace('\\','/',$reportPath));
    }

    public function getReportPath()
    {
        return $this->reportPath;
    }

    public function setLocale($locale)
    {
        $this->locale = $locale;
    }

    public function getLocale()
    {
        return $this->locale;
    }

    protected function setReportName($reportName)
    {
        $this->reportName = $reportName;
    }

    public function getReportName()
    {
        return $this->reportName;
    }

    public function getReportContent(){
        if(file_exists($this->reportPath)){
            return file_get_contents($this->reportPath);
        }
        else throw new \Exception("O relatório ainda não existe! Gere-o primeiro com generateReport().");
    }

    public function __construct($reportName=null, $locale=null, $reportFormat=null, $reportPath=null, $inputtedParameters=null){
    //public function __construct($reportName=null, $locale=null, $inputtedParameters=null, $reportPath=null){
        if($hasCode=$reportName!==null) $this->setReportName($reportName);

        if($hasLocale=$locale!==null) $this->setLocale($locale);

        if($hasReportFormat=$reportFormat!==null) $this->setReportFormat($reportFormat);

        if($hasPath=$reportPath!==null) $this->setReportPath($reportPath);

        if($hasParameters=$inputtedParameters!==null){
            $this->setInputtedParameters($inputtedParameters);
        }else{
            $this->setInputtedParameters(Array()); //atribui-se um array vazio para o foreach não soltar um erro
        }

        if(!isset(self::$REPOSITORY_PATH)){
            $db=\Setforms2\Runtime\Connection::getInstance();
            $repositoryPath=$db->fetchAll("SELECT REPORTPATH FROM PARAMGERAL");
            self::setRepositoryPath($repositoryPath[0]["REPORTPATH"]);
        }
//        if($hasCode&&$hasLocale) $this->generateReport($reportName,$locale);
    }

    //TODO: public static function quickGenerateReport($reportName=null, $inputtedParameters=null, $reportFormat=null, $reportPath=null, $locale=null){}


    public function generateReport($reportFormat=null, $locale=null, $inputtedParameters=null, $reportPath=null, $reportName=null,$showCommand=null){
    //public function generateReport($reportFormat=null, $reportName=null, $locale=null, $inputtedParameters=null, $reportPath=null){

        //o relatório pode levar algum tempo pra gerar
        set_time_limit(0);

        //checa se algum parâmetro foi passado
        //se foi, seta em this. se não, pega de this
        ($reportFormat===null) ? $reportFormat=$this->getReportFormat() : $this->setReportFormat($reportFormat);
        ($reportName===null) ? $reportName=$this->getReportName() : $this->setReportName($reportName);
        ($locale===null) ? $locale=$this->getLocale() : $this->setLocale($locale);
        ($inputtedParameters===null) ? $inputtedParameters=$this->getInputtedParameters() : $this->setInputtedParameters($inputtedParameters);
        ($reportPath===null) ? $reportPath=$this->getReportPath() : $this->setReportPath($reportPath);

        //se o caminho report não foi setado, gera o nome do arquivo pdf temporário
        if (isset($reportPath)){
            //@todo soltar um warning se $reportpath não for um caminho que existe
            $filePath=$reportPath;
        }else{ // se foi, esse é o nome do arquivo
            //sys_get_temp_dir()
            $filePath=realpath(__DIR__).'/../../../../../public/reports/'.uniqid('temp_'.$reportName.'_').'.'.strtolower($reportFormat);
//            tk_abort($filePath);
        }

        //checa se o caminho no banco aponta pra um lugar que existe
        if(!file_exists($repositoryPath=self::getRepositoryPath())){
            //se não, considera sources/reports/gestao e lança um notice
//            trigger_error("O caminho descrito em REPORTPATH de PARAMGERAL não existe. Considere modificá-lo.", E_USER_NOTICE);
            $repositoryPath=realpath(__DIR__."/../../../reports/");
        }

        if (!file_exists($repositoryPath."/".$reportName.".rptdesign")) {  //se não existe relatório não dá pra rodar
            throw new \Exception("O relatório especificado($reportName)
             não existe no caminho especificado($repositoryPath).");
        }

        $isWindows=(strtoupper(substr(PHP_OS, 0, 3)) === 'WIN'); //esse marcador vai ser usado em alguns lugares

        //monta a string que vai ser executada
        //algumas coisas no comando de geração mudam de windows pra linux

        if ($isWindows) {  //aqui se chama o java diretamente
            $javaPath=realpath(__DIR__."/../../../libraries/jre/bin/java.exe"); //caminho para o java que vem junto com o tecfood
            //checando se o java está entre as libs do tecfood (era pra estar)
            if (file_exists($javaPath)){ //se estiver, concatena na string que vai ser "exec()"ada (com mais memória disponível)
                //-Xmx1240m
                $cmdString=$javaPath.' -Xmx1024m -cp '.realpath(__DIR__."/../../../libraries/ReportEngine/lib").'\* -Duser.language=pt -Duser.country=BR -DBIRT_HOME="'.realpath(__DIR__."/../../../../libraries/ReportEngine").'" org.eclipse.birt.report.engine.api.ReportRunner -m runrender -f '.$reportFormat.' --locale '.$locale.' -o '.$filePath.'<PARAMETERS> '.$repositoryPath."/$reportName.rptdesign";
            }else{ //se não, dá pau
                throw new \Exception("O Java não foi encontrado entre as bibliotecas. Cheque sua instalação do sistema.");
            }

        }else{
            //está rodando no linux
            //aqui roda-se um bash que chama o java

            $cmdString='sh '.realpath(__DIR__."/../../../libraries/ReportEngine").'/genReport.sh -m runrender -f '.$reportFormat.' --locale '.$locale.' -o '.$filePath.'<PARAMETERS> '.$repositoryPath."/$reportName.rptdesign";
        }

//        $cmdString='java -cp '.realpath(__DIR__."/../../../libraries/ReportEngine/lib").'\* -DBIRT_HOME="'.realpath(__DIR__."/../../../../libraries/ReportEngine").'" org.eclipse.birt.report.engine.api.ReportRunner -m runrender -f PDF --locale '.$locale.' -o '.$tmpFilePath.'<PARAMETERS> '.$repositoryPath."/$reportName.rptdesign";

        $paramString="";

        foreach($inputtedParameters as $name=>$value){
                $paramString.=" -p ".$name.'='.$value;
        }

        $toExecuteNoLang=str_replace("<PARAMETERS>", $paramString, $cmdString); //coloca os parâmetros no lugar certo

        //se estiver no Windows, é só executar
        if($isWindows){
            $toExecute=$toExecuteNoLang;
        }
        //se não. é preciso mudar o locale do ambiente do exec para datas serem lidas corretamente pelo gerador
        //e passadas pro banco
        else{
            $toExecute="LANG=pt_BR.UTF-8; LC_ALL=pt_BR.UTF-8; LANGUAGE=pt_BR:pt:en; ".$toExecuteNoLang;
        }

//       $comando='c:/PROGRA~1/Java/jdk1.7.0_51/bin/java -cp C:/workfolder/v_220/products/libraries/ReportEngine/lib/* -DBIRT_HOME="C:/workfolder/v_220/products/libraries/ReportEngine" org.eclipse.birt.report.engine.api.ReportRunner -m runrender -f PDF --locale en_US -o c:/workfolder/temp_painel_GU_mod2_540dc3c400fb3_XXX.pdf -p P_CDGUNIDADE=000000000999 -p URL_LOGO_CLIENTE=http://192.168.120.215:9090/v_220/products/projects/tecfood/public/images/logo/1-logo.png -p P_NRORG=1 C:/workfolder/v_220/products/sources/reports/gestao/painel_GU_mod2.rptdesign';

        $output=Array();
        //exibe o comando para linha de execução
        //merece melhorar mas por enquanto não há melhor forma para exibir a mensagem
        if($showCommand==true){
            tk_abort($toExecute);
        }
        //tk_abort($toExecute);
        $ret = exec($toExecute,$output,$status);

        if ($status==0) {         //se o exec tiver rodado direitinho, retorna o filepath
            $this->setReportPath(str_replace('\\','/',$filePath));
            return $filePath;
        }
        else{
            $errorMsg="Aconteceu um problema e o relatório não pôde ser gerado. Cheque se o .rptdesign passado
            é válido.".
                "<br/><br/>Código de geração: ". $toExecute;
                $errorMsg=$errorMsg . ((!empty($output)) ? "<br/><br/>Saída: ". $output : "");
            throw new \Exception($errorMsg);
        }
    }

}

//$report= new Report("analise_desempenho_compras", "pt_BR");
//$report->addParameterArray(Array("P_DATAFIM" => "02/09/2014", "P_DATAINICIO" => "01/08/2013"));
//$path=$report->generateReport();
//echo $path;

//echo shell_exec('java -cp C:/workfolder/v_220/products/libraries/ReportEngine/lib/* -DBIRT_HOME="C:/workfolder/v_220/products/libraries/ReportEngine" org.eclipse.birt.report.engine.api.ReportRunner -m runrender -f PDF --locale en_US -o c:/workfolder/temp_painel_GU_mod2_540dc3c400fb3_XXX.pdf -p P_CDGUNIDADE=000000000999 -p URL_LOGO_CLIENTE=http://192.168.120.215:9090/v_220/products/projects/tecfood/public/images/logo/1-logo.png -p P_NRORG=1 C:/workfolder/v_220/products/sources/reports/gestao/painel_GU_mod2.rptdesign');

//$comando='java -cp C:/workfolder/v_220/products/libraries/ReportEngine/lib/* -DBIRT_HOME="C:/workfolder/v_220/products/libraries/ReportEngine" org.eclipse.birt.report.engine.api.ReportRunner -m runrender -f PDF --locale en_US -o c:/workfolder/temp_painel_GU_mod2_540dc3c400fb3_XXX.pdf -p P_CDGUNIDADE=000000000999 -p URL_LOGO_CLIENTE=http://192.168.120.215:9090/v_220/products/projects/tecfood/public/images/logo/1-logo.png -p P_NRORG=1 C:/workfolder/v_220/products/sources/reports/gestao/painel_GU_mod2.rptdesign';
//$pCOM = new \COM("WScript.Shell");
//$pShell = $pCOM->Exec($comando);
//echo $pShell->StdOut->ReadAll;    # Standard output
//echo $pShell->StdErr->ReadAll;    # Error