<?php

/**
 * Produto
 *
 * This class has been auto-generated by the Doctrine ORM Framework
 *
 * @package    DbModel
 * @subpackage Model
 * @author     SetForms Team <development@setforms.com>
 * @version    Release: SVN: $Id: Builder.php 6508 2009-10-14 06:28:49Z jwage $
 */
class Produto extends DaoProduto
{


    /**
     * Consulta produto.
     *
     * @param integer $nrorg     Numero da Organizacao.
     * @param integer $cdproduto Codigo do Produto.
     *
     * @return ProdutoEntity
     */
    public static function consultaProduto($nrorg, $cdproduto){
        $result = self::getObjectByWhere('NRORG = ? AND CDPRODUTO = ? ', array($nrorg, $cdproduto));
        return $result;
    }


    /**
     * Consulta registro na tabela de produto.
     *
     * @param integer $nrorg     Numero da Organiza�ao.
     * @param string  $cdarvprod Numero do codigo da arvore de produto.
     *
     * @return boolean|Ambigous <mixed, NULL>
     */
    public static function consultaArvoreProduto($nrorg, $cdarvprod){
        try {
            $result = self::getObjectByWhere(
                'NRORG = ? AND CDARVPROD = ? ',
                array(
                 $nrorg,
                 $cdarvprod,
                )
            );

        } catch (Exception $e) {
            showClassError($e, __FUNCTION__, __LINE__);
            return false;

        }
        return $result;

    }



    /**
     * Consulta o ultimo item filho de um produto pai.
     *
     * @param integer $nrorg      Numero da organizacao.
     * @param string  $cdarvprod  Codigo de arvore do produto pai.
     *
     * @return Ambigous <<type>, multitype:, mixed>
     */
    public static function consultaMaiorItemArvore($nrorg, $cdarvprod){
        $sql = "SELECT
                  MAX(SUBSTR(F.CDARVPROD, NVL(INSTR(P.CDARVPROD
                  ||'.', '.'), 0), LENGTH(F.CDARVPROD))) MAIOR
                FROM PRODUTO F
                INNER JOIN PRODUTO P
                ON P.CDPRODUTO = F.CDARVPAI
                AND P.NRORG    = $nrorg
                AND P.CDARVPROD = '$cdarvprod'
                GROUP BY P.CDARVPROD
                ";

        $result = \Setforms2\Runtime\Connection::getInstance()->fetchAll($sql);
        return $result;
    }

    public static function consultaProdutoByWhere($where)
    {
        try {
//            $obj = self::getObjectByWhere($where);
//            tk_warning($where);
            $db = \Setforms2\Runtime\Connection::getInstance();
            return $db->fetchAll('SELECT * FROM PRODUTO WHERE '.$where);
        } catch (Exception $e) {
            tk_warning($e->getMessage());
            throw $e;
        }

        //return $obj;
    }


    public static function produtoCustoLocalEstoque(
        $cdfilial,
        $cdproduto,
        $nrtipocusto,
        $dtreferencia,
        $dtatual,
        $almoxarife,
        $localestoq,
        $nrloteestq,
        $nrsublote
    ) {

        $params = array(
            $cdfilial,
            $cdproduto,
            $nrtipocusto,
            $dtreferencia,
            $dtatual,
            $almoxarife,
            $localestoq,
            $nrloteestq,
            $nrsublote
        );

        $conn = self::getConnection();
        $query =
            "
            SELECT
            CUSTO_PRODUTO_LCEST
                (
                :P_CDFILIAL,
                :P_CDPRODUTO,
                :P_NRTIPOCUSTO,
                :P_DTREFERENCIA,
                :P_DTATUAL,
                :P_ALMOXARIFE,
                :P_LOCALESTOQ,
                :P_NRLOTEESTQ,
                :P_NRSUBLOTE
                )
            FROM DUAL
            "
        ;
        $result = $conn->fetchOne($query, $params);
        return $result;



    }



    public static function consultaPeriodicidadeProduto($cdfilsoli, $cdproduto, $nrorg){
        $sql = "
            SELECT MAX(PERFILPROD.CDPERIENTR) AS PERIODICIDADE
            FROM PRODUTO,
              PRODUTO PRODPERIENTR,
              PERFILPROD,
              PERIENTR
            WHERE PRODUTO.CDPRODESTO   = :P_PRODUTO
            AND PERFILPROD.CDPRODUTO   = PRODPERIENTR.CDPRODUTO
            AND PERFILPROD.CDFILIAL    = :P_CDFILSOLI
            AND PERFILPROD.CDPERIENTR  = PERIENTR.CDPERIENTR
            AND PERFILPROD.NRORG        = :P_NRORG
            AND PRODPERIENTR.CDARVPROD =
              (SELECT MAX(B.CDARVPROD)
              FROM PERFILPROD A,
                PRODUTO B
              WHERE A.CDPRODUTO                         = B.CDPRODUTO
              AND A.NRORG                               = :P_NRORG
              AND A.NRORG                               = B.NRORG
              AND A.CDFILIAL                            = :P_CDFILSOLI
              AND INSTR(PRODUTO.CDARVPROD, B.CDARVPROD) = 1
              )
        ";

        $paramsWhere = array(
            'P_PRODUTO'   => $cdproduto,
            'P_CDFILSOLI' => $cdfilsoli,
            'P_NRORG'     => $nrorg,
        );

        $db = self::getConnection();
        return  $db->fetchAll($sql, $paramsWhere);

    }



    /**
     * Exclui o produto.
     *
     * @param integer $cdproduto C?o do produto.
     * @param integer $nrorg     N?o da Organiza?.
     *
     * @return boolean
     */
    public static function excluiFilhosItemProduto($nrorg, $cdproduto){
        try {
            $consulta = self::getObjectByWhere(
                'NRORG = ? AND CDARVPAI = ?',
                array(
                    $nrorg,
                    $cdproduto
                )
            );

            if ($consulta instanceof ProdutoEntity) {
                $produto = $consulta->getCdproduto();

                $db = self::getConnection();
                $exclusao = "DELETE FROM PRODUTO
                          WHERE NRORG = ?
                            AND CDARVPAI = ?";
                $params = array($nrorg, $cdproduto);
                $db->execute($exclusao, $params);

                self::excluiFilhosItemProduto($nrorg, $produto);
            }


        } catch(Exception $e) {
            tk_warning('Erro desconhecido'. $e->getMessage());
        }

    }


    public static function custoProduto($cdfilial, $cdproduto, $nrtipocusto, $data){

        $sql =
        "
            SELECT NVL(CUSTO_PRODUTO (:P_CDFILIAL, :P_CDPRODUTO, :P_NRTIPOCUSTO, :P_DATA, SYSDATE), 0) AS CUSTO
            FROM DUAL
        ";
        $paramsWhere = array(
            'P_CDFILIAL'    => $cdfilial,
            'P_CDPRODUTO'   => $cdproduto,
            'P_NRTIPOCUSTO' => $nrtipocusto,
            'P_DATA'        => $data,
        );

        $db = self::getConnection();
        $resultado = $db->fetchAll($sql, $paramsWhere);

        return $resultado;
    }


    public static function consultaFilhosProduto($nrorg, $cdarvprod){
        try {
            $sql = "SELECT * FROM PRODUTO WHERE NRORG = '$nrorg' AND CDARVPROD LIKE '$cdarvprod%'";
            $db = self::getConnection();
            $result = $db->fetchAll($sql);

            return $result;
        } catch (Exception $e) {
            tk_warning($e->getMessage());
            return false;

        }
    }



}//end class